var inquirer = require('inquirer');
var path = require('path');
const fs = require('fs')
const fse = require('fs-extra');
const chalk = require('chalk');
const { appComponents, appPackage, templates } = require('./paths');

const defaultValues = {
    module_name: 'text',
    name: '文本',
}
const templateValues = [
    {
        name: 'React演示',
        value: 'basic'
    }, {
        name: 'Three Component',
        value: 'three-component'
    }
]
module.exports = (cb) => {
    inquirer
        .prompt([
            {
                type: 'input',
                name: 'moduleName',
                message: "你的组件名称是什么?",
                validate: function (value) {
                    return /^[a-zA-Z][\w-]+$/.test(value) ? true : '名称只能是 英文开头 包含 英文、数字、_- 的字符串'
                },
                default: defaultValues.module_name
            },
            {
                type: 'input',
                name: 'name',
                message: "中文别名是什么?",
                default: defaultValues.name
            },
            { type: 'list', name: 'template', message: '要添加哪种模版?', choices: templateValues }
        ])
        .then(answers => {
            let { moduleName, name, template } = answers;
            const base = {
                name,
                module_name: moduleName,
                version: '1.0.0',
                show: 1,
            }

            const copyPath = path.join(appComponents, moduleName)
            // 检查重名
            if (fs.existsSync(copyPath)) {
                console.log(chalk.red(`组件名称已存在`))
                process.exit()
            }

            // 重写配置
            const templatePath = path.join(templates, template);
            const configPath = 'config/main.json';

            fse.copySync(templatePath, copyPath, {
                filter: (src, dest) => {
                    if (src === path.join(templatePath, configPath)) {
                        const config = require(src)
                        config.base = base;
                        fs.writeFileSync(path.join(copyPath, configPath), JSON.stringify(config, null, 2))
                        return 0;
                    }
                    return 1;
                },
            });

            console.log()
            console.log(chalk.green(`模版添加成功`))
            cb && cb()
        })
        .catch((error) => cb && cb(error));
}