const path = require('path');
const { appSrc, ownLoaders, cliNodeModules } = require('./paths');

module.exports = function ({
    isProduction,
    uniqueId,
    entrys
}) {
    return {
        stats: 'errors-only',
        module: {
            rules: [
                // 暴露变量到全局
                !isProduction && {
                    test: /^three$/,
                    loader: 'expose-loader',
                    options: {
                        exposes: {
                            globalName: 'THREE',
                            override: true
                        }
                    },
                },

                {
                    test: /\.(js|mjs|jsx|ts|tsx)$/,
                    include: appSrc,
                    loader: require.resolve('babel-loader'),
                    options: {
                        customize: require.resolve(
                            'babel-preset-react-app/webpack-overrides'
                        ),
                        presets: [require.resolve('babel-preset-react-app')],
                        plugins: [
                            [
                                require.resolve('babel-plugin-named-asset-import'),
                                {
                                    loaderMap: {
                                        svg: {
                                            ReactComponent:
                                                '@svgr/webpack?-svgo,+titleProp,+ref![path]',
                                        },
                                    },
                                },
                            ],
                        ],
                        cacheDirectory: false,
                        cacheCompression: false,
                    },
                },
                {
                    // 修改入口文件
                    test: Object.values(entrys),    // 仅匹配所有组件的index文件
                    loader: 'react-entry-loader',
                    options: {
                        isBuild: isProduction,
                        uniqueId
                    }
                },

                {
                    test: /\.js\.map$/,
                    use: {
                        loader: 'file-loader',
                    },
                },

                {
                    test: /\.css$/,
                    oneOf: [
                        {
                            test: /\.(M|m)odule\.css$/,
                            use: [
                                {
                                    loader: 'style-loader',
                                },
                                {
                                    loader: 'css-loader',
                                    options: {
                                        modules: {
                                            localIdentName: isProduction ? '[name]_[local]_[hash:base64:5]' : '[path][name]__[local]',
                                        }
                                    },
                                },
                            ]
                        },
                        {
                            use: ['style-loader', 'css-loader']
                        }
                    ]
                },
                {
                    test: /\.less$/,
                    include: [appSrc],
                    use: [
                        {
                            loader: 'style-loader',
                        },
                        {
                            loader: 'css-loader',
                            options: {
                                modules: true,
                            },
                        },
                        {
                            loader: 'less-loader',
                        },
                    ],
                },
                {
                    // exclude: /node_modules/,
                    test: /\.(png|jpg|jpeg|gif|svg|woff|woff2|ttf|eot)(\?.*$|$)/,
                    loader: 'url-loader',
                    options: {
                        limit: 20 * 1024 * 1024,
                    }
                },
            ].filter(Boolean),
        },

        resolveLoader: {
            modules: [
                'node_modules',
                ownLoaders,
                cliNodeModules,
            ]
        },

        resolve: {
            extensions: ['.js', '.jsx'],
            modules: [
                'node_modules',
                cliNodeModules
            ]
        },

    }
}