const HtmlPlugin = require('html-webpack-plugin');
const path = require('path');
const fs = require('fs');
const commonConfig = require('./webpack.config.common');
const webpack = require('webpack');

const { appPackage, lib, public, webpackConfigExtend, cliNodeModules } = require('./paths');

let webpackConfigExtendFn;
if (fs.existsSync(webpackConfigExtend)) {
  webpackConfigExtendFn = require(webpackConfigExtend);
}

process.env.NODE_ENV = 'development';

module.exports = ({ port, modules }) => {

  const entrys = modules.reduce((acc, item) => {
    acc[item.name] = path.resolve(item.path)
    return acc
  }, {})

  const cf = {
    ...commonConfig({
      isProduction: false,
      entrys
    }),

    mode: 'development',
    entry: entrys,
    output: {
      filename: '[name].js',
      globalObject: 'this',
      libraryTarget: 'var',
      library: '__MAIN__' // 暴露到window上的属性名称
    },
    devtool: 'source-map',
    plugins: [
      new webpack.ProvidePlugin({
        THREE: 'three',
        React: 'react',
        ReactDom: 'react-dom',
      }),
      new HtmlPlugin({
        filename: 'index.html',
        template: path.join(public, 'home.ejs'),
        templateParameters: {
          'modules': Object.keys(entrys).toString()
        },
        chunks: ['']
      }),
      ...Object.keys(entrys).map(name => {
        return new HtmlPlugin({
          filename: `${name}.html`,
          template: path.join(public, 'component.ejs'),
          templateParameters: {
            "module_name": name,
            "assets": appPackage.easyv.assets,
          },
          chunks: [name]
        })
      })
    ],
    externals: {
      'react': 'react',
      'react-dom': 'ReactDOM'
    },

    devServer: {
      port,
      hot: false,
      quiet: true,
      before: function (app, server, compiler) {
        var multer = require('multer')
        var storage = multer.memoryStorage()
        var upload = multer({ storage: storage })
        const fileCache = new Map();
        // 用户资源上传
        app.post('/__develop__/assets', upload.single('file'), function (req, res) {
          const a = req.file;
          fileCache.set(a.originalname, a.buffer)
          res.json({ url: `${req.url}/${a.originalname}` });
        });
        // 用户资源获取
        app.get('/__develop__/assets/:name', function (req, res) {
          const file = fileCache.get(req.params.name);
          if (!file) return res.status(404);
          res.send(file)
        });
        // 静态资源
        app.get('/static/*', function (req, res) {
          let filepath;
          [cliNodeModules, lib].find((root) => {
            const p = req.url.replace('/static', root);
            if (fs.existsSync(p)) {
              filepath = p;
              return true;
            }
          })
          if (!filepath) return res.status(404).end();
          fs.readFile(filepath, (err, data) => {
            if (err) { return res.status(404).end() }
            res.send(data)
          })
        });
      }
    }
  }

  if (webpackConfigExtendFn) {
    return webpackConfigExtendFn(cf, {
      production: false
    })
  }

  return cf;
};
